local E, L, V, P, G = unpack(ElvUI)
local IF = E:NewModule("InfoBar", "AceEvent-3.0", "AceHook-3.0", "AceConsole-3.0", "AceTimer-3.0")
local LDB = LibStub:GetLibrary("LibDataBroker-1.1")
local LibQTip = LibStub("LibQTip-1.0")
local EP = LibStub("LibElvUIPlugin-1.0");
local addon, ns = ...
IF.modName = L["InfoBar"]

local maxMenuButtons, infobarTypes, usedInfoBar = 20, {}, {}
local brokerTooltip

-- Constants
IF.classColor = E.myclass == 'PRIEST' and E.PriestColors or (CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[E.myclass] or RAID_CLASS_COLORS[E.myclass])
IF.version = GetAddOnMetadata("ElvUI_InfoBar", "Version")

IF.class = {
	["DEATHKNIGHT"] = { 0.77,	0.12,		0.23 },
	["DEMONHUNTER"] = { 0.64,	0.19,		0.79 },
	["DRUID"]       = { 1,		0.49,		0.04 },
	["HUNTER"]      = { 0.58,	0.86,		0.49 },
	["MAGE"]        = { 0.2,	0.76,		1 },
	["MONK"]        = { 0,		1,			0.59 },
	["PALADIN"]     = { 0.96,	0.55,		0.73 },
	["PRIEST"]      = { 0.8,	0.87,		0.9 },
	["ROGUE"]       = { 1,		0.96,		0.41 },
	["SHAMAN"]      = { 0,	    0.44,		0.87 },
	["WARLOCK"]     = { 0.6,	0.47,		0.85 },
	["WARRIOR"]     = { 0.9,	0.65,		0.45 },
}

for class, color in pairs(IF.class) do
	IF.class[class] = { r = color[1], g = color[2], b = color[3] }
end

function IF:Print(...)
	print("|cfff960d9".."InfoBar:|r", ...)
end

local function GetInfoBarList()
    if not E.db["InfoBar"] then
        E.db["InfoBar"] = {}
    end
    if not E.db["InfoBar"]["List"] then
        E.db["InfoBar"]["List"] = {
            "Memory",
            "System",
            "Talent",
            "Durability",
            "Friends",
            "Guild",
            "Money",
            "WorldQuests",
        }
    end
    return E.db["InfoBar"]["List"]
end

local function OpenMenu(infobar)
    local padding, numShown = 20, 0
    local menu = IF_Menu
    local oldRef = menu.ref

    if oldRef and oldRef ~= infobar and not oldRef.infobarType then
        menu.ref:SetAlpha(0)
    end

    for i = 1, maxMenuButtons do
        local button = menu["Button"..i]
        button:Hide()
    end

    for infobarType, info in pairs(infobarTypes) do
        local isUsed = usedInfoBar[infobarType]

        if not isUsed and numShown <= maxMenuButtons then
            numShown = numShown + 1

            -- Add InfoBar Button to the menu
            local button = menu["Button"..numShown]

            button.ref = infobar
            button.infobarType = infobarType
            button.info = info
            button:SetNormalTexture(info.icon or "Interface\\Icons\\inv_misc_questionmark")
            button:SetText(info.title)
            button:Show()
        end
    end

    if numShown == 0 then
        padding = padding - 10
    end

    if infobar.infobarType then
        padding = padding + 35
        menu.Clear:Show()
    else
        menu.Clear:Hide()
    end

    menu.ref = infobar
    menu:SetHeight(25 * numShown + padding)
    menu:ClearAllPoints()
	if E.db.InfoBar.Position == "BOTTOM" then 
		menu:SetPoint("BOTTOM", infobar, "TOP", 0, 5)
	else
		menu:SetPoint("TOP", infobar, "BOTTOM", 0, 5)
	end
    menu:Show()

    GameTooltip_Hide()
end

local function SetButton(button,index,infobarType,info,isInit)
    button:SetAlpha(1)
    button.interval = info.interval or 1
    button.isBroker = info.isBroker
    button.title = info.title
    button.infobarType = infobarType
    button.clickFunc = info.clickFunc
    button.onUpdate = info.onUpdate
    button.tooltipFunc = info.tooltipFunc
    button.onLeaveFunc = info.onLeaveFunc
    button.eventFunc = info.eventFunc
    button.registerLDBEvent = info.registerLDBEvent
    button.unregisterLDBEvent = info.unregisterLDBEvent
    button:SetText(info.title)

    usedInfoBar[infobarType] = index

    if type(info.events) == "table" then
        for _, event in pairs(info.events) do
            if event == "UNIT_AURA" or event == "UNIT_RESISTANCES" or event == "UNIT_STATS" or event == "UNIT_ATTACK_POWER"
            or event == "UNIT_RANGED_ATTACK_POWER" or event == "UNIT_TARGET" or event == "UNIT_SPELL_HASTE" then
                button:RegisterUnitEvent(event, "player")
            elseif event == 'COMBAT_LOG_EVENT_UNFILTERED' then
                button:RegisterUnitEvent(event, UnitGUID("player"), UnitGUID("pet"))
            else
                button:RegisterEvent(event)
            end
        end
    end

    if button.registerLDBEvent then
        button:registerLDBEvent()
    end
    -- Save Setting
    if not isInit then
        E.db["InfoBar"]["List"][index] = infobarType
    end

    if info.initFunc then
        info.initFunc(button)
    end

    if info.icon then
        button:SetNormalTexture(info.icon)
        button.Text:SetPoint("TOPLEFT",28,-8)
        button.Highlight:Point("TOPLEFT",28,-8)
        button.Icon:Show()
		button.Icon:CreateBackdrop()
    else
        button.Text:SetPoint("TOPLEFT",8,-8)
        button.Highlight:Point("TOPLEFT",8,-8)
        button.Icon:Hide()
    end

    button.Highlight:Point("BOTTOMRIGHT",-8,8)
end

function IF_Button_OnClick(self, button)
    if button == "RightButton" and not IsShiftKeyDown() and not IF_Menu:IsShown() then
        IF_Button_OnLeave(self)
        for _, tooltip in LibQTip:IterateTooltips() do
            tooltip:Hide()
        end
        OpenMenu(self)
        return
	else
		IF_Menu:Hide()
    end

    if button == "LeftButton" and self.clickFunc then
        self.clickFunc(self, button)
        return
    end

    if button == "RightButton" and IsShiftKeyDown() and self.clickFunc then
        self.clickFunc(self, button)
        return
    end
end

function IF_Button_OnEnter(self)
    local menu = IF_Menu

    if self.tooltipFunc and not menu:IsShown() then
        self.tooltipFunc(self)
    elseif not self.infobarType then
		if E.db.InfoBar.Position == "BOTTOM" then
			GameTooltip:SetOwner(self, "ANCHOR_TOP", 0, 0)
		else
			GameTooltip:SetOwner(self, "ANCHOR_BOTTOM", 0, 0)
		end
        GameTooltip:AddLine(L["Right-Click to open infobar list"],1,1,1)
        GameTooltip:Show()
    end

    if self.infobarType then
        self:SetAlpha(1)
    else
        self:SetAlpha(0.5)
    end
end

function IF_Button_OnLeave(self)
    local menu = IF_Menu

    if self.infobarType then
        self:SetAlpha(1)
    elseif menu:IsShown() and menu.ref == self then
        self:SetAlpha(0.5)
    else
        self:SetAlpha(0)
    end

    if self.onLeaveFunc then
        C_Timer.After(0.25, function() self.onLeaveFunc(self) end)
    end

    if brokerTooltip then
        brokerTooltip:Hide()
    end
    GameTooltip_Hide()
end

function IF_Button_OnUpdate(self, elapsed)
    if self.onUpdate then
        self.update = self.update + elapsed

        if self.update > self.interval then
            self.onUpdate(self)
            self.update = 0
        end
    end
end

function IF_Button_OnEvent(self, event, ...)
    if self.eventFunc then
        self.eventFunc(self, event, ...)
    end
end

function IF_Button_OnReset(self,noSave)
    local dataBase = GetInfoBarList()

    if self.infobarType then
        usedInfoBar[self.infobarType] = nil
    end
    if not noSave then
        dataBase[self:GetID()] = nil
    end

    self.infobarType = nil
    self.clickFunc = nil
    self.onUpdate = nil
    self.tooltipFunc = nil
    self.Text:SetPoint("TOPLEFT",8,-8)
    self.Highlight:Point("TOPLEFT",8,-8)
    self.Highlight:Point("BOTTOMRIGHT",-8,8)
    self:SetText("")
    self:SetNormalTexture("")
    self.Icon:Hide()
    self:SetAlpha(0)
    self:UnregisterAllEvents()
    if self.unregisterLDBEvent then self:unregisterLDBEvent() end

    IF_Menu:Hide()
end

function IF_Menu_OnInit(self)
    local dataBase = GetInfoBarList()

    IF_RegisterLDB()

    for index, infobarType in pairs(dataBase) do
        local button = _G["IF_InfoBar"..index]
        local info = infobarTypes[infobarType]

        if info then
            SetButton(button,index,infobarType,info,true)
        end
    end

    self:UnregisterEvent("PLAYER_LOGIN")
end

function IF_MenuButton_OnClick(self)
    local button, infobarType, info = self.ref, self.infobarType, self.info
    local index = button:GetID()

    -- Free Prev InfoBar
    if button.infobarType then
        usedInfoBar[button.infobarType] = nil
    end

    if button.unregisterLDBEvent then
        button:unregisterLDBEvent()
    end

    SetButton(button,index,infobarType,info)

    IF_Menu:Hide()
end

function IF_RegisterLDB()
    for name, obj in LDB:DataObjectIterator() do
        local info = {}

        local longValue, brokerTitle, brokerValue
        local OnEnter = nil
        local OnLeave = nil
        local curFrame = nil
        if obj.OnTooltipShow then
            function OnEnter(self)
                if brokerValue then
                    brokerTooltip:SetOwner(self, "ANCHOR_TOP", 0, 0)
                    brokerTooltip:AddLine(brokerTitle, 1, 1, 1)
                    brokerTooltip:SetPrevLineJustify("CENTER")
                    brokerTooltip:AddDivider()
                    brokerTooltip:AddLine(brokerValue, 1, 1, 1)
                    brokerTooltip:Show()
                end
                GameTooltip:SetOwner(brokerTooltip, "ANCHOR_TOP", 0, 0)
                obj.OnTooltipShow(GameTooltip)
                GameTooltip:Show()
            end
        end

        if obj.OnEnter then
            function OnEnter(self)
                if brokerValue then
                    brokerTooltip:SetOwner(self, "ANCHOR_TOP", 0, 0)
                    brokerTooltip:AddLine(brokerTitle, 1, 1, 1)
                    brokerTooltip:SetPrevLineJustify("CENTER")
                    brokerTooltip:AddDivider()
                    brokerTooltip:AddLine(brokerValue, 1, 1, 1)
                    brokerTooltip:Show()
                    obj.OnEnter(brokerTooltip)
                else
                    obj.OnEnter(self)
                end
                GameTooltip_Hide()
            end
        end

        if obj.OnLeave then
            function OnLeave(self)
                obj.OnLeave(self)
            end
        end

        local function OnClick(self, button)
            obj.OnClick(self, button)
        end

        local function textUpdate(event, name, key, value, dataobj)
            if value == nil or (strlen(value) >= 3) or value == "n/a" or name == value then
                if value and strlen(value) > 30 then
                    longValue = true
                    brokerTitle = name
                    brokerValue = value
                    curFrame:SetText(name)
                else
                    longValue = false
                    curFrame:SetText(value ~= "n/a" and value or name)
                end
            else
                curFrame:SetFormattedText("%s: |cffFFFFFF%s|r", name, value)
            end
        end

        local function registerEvent(self)
            curFrame = self
            LDB.RegisterCallback(self,"LibDataBroker_AttributeChanged_"..name.."_text", textUpdate)
            LDB.RegisterCallback(self,"LibDataBroker_AttributeChanged_"..name.."_value", textUpdate)
            LDB.callbacks:Fire("LibDataBroker_AttributeChanged_"..name.."_text", name, nil, obj.text, obj)
        end

        local function unregisterEvent(self)
            curFrame = self
            LDB.UnregisterCallback(self,"LibDataBroker_AttributeChanged_"..name.."_text")
            LDB.UnregisterCallback(self,"LibDataBroker_AttributeChanged_"..name.."_value")
        end

        info.isBroker = true
        info.title = name
        info.icon = obj.icon
        info.clickFunc = OnClick
        info.tooltipFunc = OnEnter
        info.onLeaveFunc = OnLeave
        info.registerLDBEvent = registerEvent
        info.unregisterLDBEvent = unregisterEvent

        IF:RegisterInfoBarType(name, info)
    end
end

function IF:RegisterInfoBarType(infobarType, infobarInfo)
    infobarTypes[infobarType] = infobarInfo
end

function IF:Initialize()
    local tooltipFont = CreateFont("IF_TooltipFont")
    tooltipFont:FontTemplate(E["media"].normFont, 11, "OUTLINE")
    brokerTooltip = CreateFrame("GameTooltip", "IF_BrokerTooltip", E.UIParent, "GameTooltipTemplate")
    local r, g, b = IF.classColor.r, IF.classColor.g, IF.classColor.b
    local font = CreateFont("IF_Font")
    font:FontTemplate(E["media"].normFont, 11, "OUTLINE")
    font:SetTextColor(1, 1, 1)
    font:SetShadowColor(0, 0, 0)
    font:SetShadowOffset(E.mult, -E.mult)

    for i = 1, maxMenuButtons do
        local infoBar = CreateFrame("Button", "IF_InfoBar"..i, E.UIParent, "IF_ButtonTemplate")
        infoBar:SetNormalFontObject(font)
        infoBar:SetSize(140 * E.db.InfoBar.Scale, 35 * E.db.InfoBar.Scale)
		infoBar:SetScale(E.db.InfoBar.Scale)
        infoBar:SetID(i)
        if i == 1 then
			if E.db.InfoBar.Position == "BOTTOM" then
				infoBar:Point("BOTTOMLEFT", E.UIParent, "BOTTOM", -140 * 4 - 6 * 3 - 3, -5)
			else
				infoBar:Point("TOPLEFT", E.UIParent, "TOP", -140 * 4 - 6 * 3 - 3, -5)
			end
        else
            infoBar:Point("LEFT", _G["IF_InfoBar"..i-1], "RIGHT", 6, 0)
        end
        infoBar.Highlight:SetTexture("Interface\\AddOns\\ElvUI_InfoBar\\media\\threat")
        infoBar.Highlight:SetBlendMode("ADD")
        infoBar.Highlight:SetVertexColor(r, g, b, .4)
		if E.db.InfoBar.Position == "TOP" then
			infoBar.Highlight:SetTexCoord(0, 1, 1, 0)
		end
        infoBar.Background = CreateFrame("Frame", nil, infoBar)
        infoBar.Background:SetFrameLevel(0)
		if E.db.InfoBar.Position == "BOTTOM" then
			infoBar.Background:Point("BOTTOMLEFT", 8, 8)
			infoBar.Background:Point("TOPRIGHT", -8, -18)
		else
			infoBar.Background:Point("TOPRIGHT", -8, -8)
			infoBar.Background:Point("BOTTOMLEFT", 8, 18)
		end
		infoBar.Background:SetTemplate("Transparent")
    end

    for i = 1, maxMenuButtons do
        local button = IF_Menu["Button"..i]
        button:SetNormalFontObject(font)
        button:SetID(i)
        button.Highlight:SetTexture("Interface\\AddOns\\ElvUI_InfoBar\\media\\threat")
        button.Highlight:SetBlendMode("ADD")
        button.Highlight:SetVertexColor(r, g, b, .4)
		if E.db.InfoBar.Position == "TOP" then
			button.Highlight:SetTexCoord(0, 1, 1, 0)
		end
        button.Background = CreateFrame("Frame", nil, button)
        button.Background:SetFrameLevel(0)
        button.Background:SetInside(button, 8, 8)
        button.Background:SetTemplate("Transparent")

        if i > 1 then button:Point("TOP", IF_Menu["Button"..i-1], "BOTTOM", 0, 10) end
    end

    local clear = IF_Menu.Clear
    clear:SetNormalFontObject(font)
    clear:SetText(L["Clear"])
    clear.Highlight:SetTexture("Interface\\AddOns\\ElvUI_InfoBar\\media\\threat")
    clear.Highlight:SetBlendMode("ADD")
    clear.Highlight:SetVertexColor(r, g, b, .4)
	if E.db.InfoBar.Position == "TOP" then
		clear.Highlight:SetTexCoord(0, 1, 1, 0)
	end
    clear.Background = CreateFrame("Frame", nil, clear)
    clear.Background:SetFrameLevel(0)
    clear.Background:SetInside(clear, 8, 8)
    clear.Background:SetTemplate("Transparent")
	
	EP:RegisterPlugin(addon, IF.AddOptions)
	
	if E.db.InfoBar.LoginMsg then
		print(L["|cff00ff96Info Bar|r "]..format("v|cff00ff96%s|r",IF.version)..L[" is loaded. Thank you for using it."])
	end
end

E:RegisterModule(IF:GetName())