local E, L, V, P, G = unpack(ElvUI)
local IF = E:GetModule("InfoBar")

--Lua functions
local select = select
local format, join = string.format, string.join
local ceil = math.ceil
local strform = string.format
local tonumber = tonumber
local tostring = tostring

--WoW API / Variables
local IsShiftKeyDown = IsShiftKeyDown

local volumeCVars = {
	{
		Name = "Master",
		Volume = {
			CVar = "Sound_MasterVolume",
			Value = 0
		},
		Enable = {
			CVar = "Sound_EnableMaster",
			Value = 0
		}
	},
	{
		Name = "SFX",
		Volume = {
			CVar = "Sound_SFXVolume",
			Value = 0
		},
		Enable = {
			CVar = "Sound_EnableSXF",
			Value = 0
		}
	},
	{
		Name = "Ambience",
		Volume = {
			CVar = "Sound_AmbienceVolume",
			Value = 0
		},
		Enable = {
			CVar = "Sound_EnableAmbience",
			Value = 0
		}
	},
	{
		Name = "Dialog",
		Volume = {
			CVar = "Sound_DialogVolume",
			Value = 0
		},
		Enable = {
			CVar = "Sound_EnableDialog",
			Value = 0
		}
	},
	{
		Name = "Music",
		Volume = {
			CVar = "Sound_MusicVolume",
			Value = 0
		},
		Enable = {
			CVar = "Sound_EnableMusic",
			Value = 0
		}
	}
}

local activeVolumeIndex = 1
local activeVolume = volumeCVars[activeVolumeIndex]


local function roundVal(num, numDecimalPlaces)
  local mult = 10^(numDecimalPlaces or 0)
  return ceil(num * mult + 0.5) / mult
end

local function Volume_OnEvent(self, event, ...)
	for key,value in pairs(volumeCVars) do
		value.Volume.Value = roundVal(GetCVar(value.Volume.CVar), 3)
		SetCVar(value.Volume.CVar, value.Volume.Value)
		value.Enable.Value = GetCVar(value.Enable.CVar)
	end

	if (event == "PLAYER_ENTERING_WORLD" ) then

		self:EnableMouseWheel(true)

		self:SetScript("OnMouseWheel", function(tself, delta)
			local vol = activeVolume.Volume.Value;
			local volScale = 100;
			
			if (IsShiftKeyDown()) then
				volScale = 10;
			end

			vol = vol + (delta / volScale)

			if (vol >= 1) then
				vol = 1
			elseif (vol <= 0) then
				vol = 0
			end
			
			
			activeVolume.Volume.Value = vol
			SetCVar(activeVolume.Volume.CVar, vol)
			self:SetText(activeVolume.Name..": "..strform("%.f", vol * 100) .. "%")
		end)

	end
	
	self:SetText(activeVolume.Name..": "..strform("%.f", activeVolume.Volume.Value * 100) .. "%")
end

local function Volume_OnEnter(self)
	if E.db.InfoBar.Position == "BOTTOM" then
		GameTooltip:SetOwner(self, "ANCHOR_TOP", 0, 0)
	else
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOM", 0, 0)
	end
	GameTooltip:AddLine(L["Volume"],1,1,1)
    GameTooltip:SetPrevLineJustify("CENTER")
    GameTooltip:AddDivider()
	
	for key,value in pairs(volumeCVars) do
		GameTooltip:AddDoubleLine(value.Volume.CVar, (value.Volume.Value * 100).."%", 1, 1, 1, 1, 1, 1)
	end
	
	GameTooltip:AddDivider()
	GameTooltip:AddLine(L["|cFF66FF00Click|r to change volume channel."],1,1,1)
	GameTooltip:AddLine(L["|cFF66FF00Scrollwheel|r to change the volume value."],1,1,1)
	GameTooltip:AddLine(L["|cFF66FF00Scrollwheel + Shift|r to change the volume value in a factor of 10."], 1, 1, 1)
	GameTooltip:Show()
end

local function Volume_OnClick(self)
	activeVolumeIndex = activeVolumeIndex + 1
	if (activeVolumeIndex == 6) then
		activeVolumeIndex = 1
	end

	activeVolume = volumeCVars[activeVolumeIndex]
	

	Volume_OnEvent(self, nil, nil)
end

do -- Initialize
    local info = {}

    info.title = L["Volume"]
    info.icon = "Interface\\AddOns\\ElvUI_InfoBar\\media\\volume.blp"
    info.clickFunc = Volume_OnClick
    info.events = { "PLAYER_ENTERING_WORLD", "CVAR_UPDATE" }
    info.eventFunc = Volume_OnEvent
    info.initFunc = Volume_OnEvent
    info.tooltipFunc = Volume_OnEnter

    IF:RegisterInfoBarType("Volume", info)
end