local E, L, V, P, G = unpack(ElvUI)
local IF = E:GetModule("InfoBar")

local questList, mapList = {}, {
	-- Legion
	[C_Map.GetMapInfo(630).name] = 630,  -- Azsuna
	[C_Map.GetMapInfo(646).name] = 646,  -- Broken Shore
	[C_Map.GetMapInfo(625).name] = 625,  -- Dalaran
	[C_Map.GetMapInfo(650).name] = 650,  -- Highmountain
	[C_Map.GetMapInfo(652).name] = 652,  -- Thunder Totem (Highmountain)
	[C_Map.GetMapInfo(634).name] = 634,  -- Stormheim
	[C_Map.GetMapInfo(680).name] = 680,  -- Suramar
	[C_Map.GetMapInfo(641).name] = 641,  -- Val'sharah
	[C_Map.GetMapInfo(885).name] = 885,  -- Antoran Wastes
	[C_Map.GetMapInfo(830).name] = 830,  -- Krokuun
	[C_Map.GetMapInfo(882).name] = 882,  -- Mac'Aree
	
	-- Battle for Azeroth
	[C_Map.GetMapInfo(14).name] = 14,  -- Arathi Highlands
	[C_Map.GetMapInfo(1161).name] = 1161,  -- Boralus (Alliance City)
	[C_Map.GetMapInfo(62).name] = 62,  -- Darkshore
	[C_Map.GetMapInfo(1165).name] = 1165,  -- Dazar'alor (Horde City)
	[C_Map.GetMapInfo(896).name] = 896,  -- Drustvar
	[C_Map.GetMapInfo(863).name] = 863,  -- Nazmir
	[C_Map.GetMapInfo(942).name] = 942,  -- Stormsong Valley
	[C_Map.GetMapInfo(895).name] = 895,  -- Tiragarde Sound
	[C_Map.GetMapInfo(864).name] = 864,  -- Vol'dun
	[C_Map.GetMapInfo(862).name] = 862,  -- Zuldazar
}

local function SortQuest(a,b)
	local an = C_TaskQuest.GetQuestInfoByQuestID(a.questId)
	local at = C_TaskQuest.GetQuestTimeLeftMinutes(a.questId)
	local bn = C_TaskQuest.GetQuestInfoByQuestID(b.questId)
	local bt = C_TaskQuest.GetQuestTimeLeftMinutes(b.questId)

	if at ~= bt then
		return at < bt
	else
		return an < bn
	end
end

local function GetTimerString(value)
	local string;
	
	if value > 60*24 then
		string = format(DAYS_ABBR, math.floor (value / (60*24)))
	elseif value > 60 then
		string = format(HOURS_ABBR, math.floor(value / 60))
	else
		string = format(MINUTES_ABBR, value)
	end
	
	return string
end

local function WorldQuests_OnEnter(self)
	local preMapID =  C_Map.GetBestMapForUnit("player")

	if E.db.InfoBar.Position == "BOTTOM" then
		GameTooltip:SetOwner(self, "ANCHOR_TOP", 0, 0)
	else
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOM", 0, 0)
	end
	
	-- complete list
	if self.isShift then
		local index = 1
		
		for name, mapID in pairs(mapList) do
			local header
			
			QuestMapFrame:GetParent():SetMapID(mapID)

			questList = C_TaskQuest.GetQuestsForPlayerByMapID(mapID)
			
			table.sort(questList, SortQuest)

			for i, info in pairs(questList) do
				local questID = info.questId
				local timeOut = C_TaskQuest.GetQuestTimeLeftMinutes(questID)

				if timeOut and timeOut > 0 then
					if not header then
						if index > 1 then
							GameTooltip:AddDivider()
						end
						
						GameTooltip:AddLine(name,1,1,1)
						GameTooltip:SetPrevLineJustify("CENTER")
						GameTooltip:AddDivider()
						
						header = true
					end
				
					local title, faction = C_TaskQuest.GetQuestInfoByQuestID(questID)
					local rarity = select(4, GetQuestTagInfo(questID))
					local color = WORLD_QUEST_QUALITY_COLORS[rarity]
					
					if faction then
						faction = GetFactionInfoByID(faction)
					else
						faction = ""
					end
					
					if color then
						title = E:RGBToHex(color.r,color.g,color.b)..title.."|r"
					end
					
					timeOut = GetTimerString(timeOut)
					
					GameTooltip:AddDoubleLine(title, timeOut, 1,1,1, 1,1,1)
				end
			end
			
			index = index + 1
		end
	end

	-- current zone (default)
	if not self.isShift then
		if not WorldMapFrame:IsShown() then
			local mapID = C_Map.GetBestMapForUnit("player")
			WorldMapFrame:SetMapID(mapID)
		end
		
		local mapID =  C_Map.GetBestMapForUnit("player")
		local name = C_Map.GetMapInfo(mapID).name
		
		if mapList[name] then
			questList = C_TaskQuest.GetQuestsForPlayerByMapID(mapID)

			table.sort(questList, SortQuest)
			
			GameTooltip:AddLine(name, 1,1,1)
			GameTooltip:SetPrevLineJustify("CENTER")
			GameTooltip:AddDivider()
				
			for i, info in ipairs(questList) do
				local questID = info.questId
				local timeOut = C_TaskQuest.GetQuestTimeLeftMinutes(questID)

				if timeOut and timeOut > 0 then
					local title, faction = C_TaskQuest.GetQuestInfoByQuestID(questID)
					local rarity = select(4, GetQuestTagInfo(questID))
					local color = WORLD_QUEST_QUALITY_COLORS[rarity]
					
					if faction then
						faction = GetFactionInfoByID(faction)
					else
						faction = ""
					end
					
					if color then
						title = E:RGBToHex(color.r,color.g,color.b)..title.."|r"
					end
					
					timeOut = GetTimerString(timeOut)
					
					GameTooltip:AddDoubleLine(title, timeOut, 1,1,1, 1,1,1)
				end
			end
		else
			--GameTooltip:AddLine("Unable to retrieve world quests for this zone.",1,0,0)
			GameTooltip:AddLine(SPELL_FAILED_INCORRECT_AREA, 1,0,0)
		end
	end
	
	QuestMapFrame:GetParent():SetMapID(preMapID)
	
	--GameTooltip:AddDivider()
	--GameTooltip:AddLine(L["|cFF66FF00Click|r to switch display"], 1,1,1)	
	--GameTooltip:SetPrevLineJustify("CENTER")
	GameTooltip:Show()
end

local function WorldQuests_OnClick(self)
	--[[
	if self.isShift then
		self.isShift = false
	else
		self.isShift = true
	end
	]]
	GameTooltip_Hide()
	WorldQuests_OnEnter(self)
end

do	-- Initialize
	local info = {}

	info.title = TRACKER_HEADER_WORLD_QUESTS
	info.icon = "Interface\\Icons\\achievement_quests_completed_07"
	info.tooltipFunc = WorldQuests_OnEnter
	info.clickFunc = WorldQuests_OnClick
	
	IF:RegisterInfoBarType("WorldQuests", info)
end