local E, L, V, P, G = unpack(ElvUI)
local IFTT = E:NewModule("InfoBarTooltip", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0")

local function CreateDivider(self,line)
    self.divider[line] = CreateFrame("Frame",nil,self)

    local divider = self.divider[line]
    divider:Height(1)
    divider.tex = divider:CreateTexture(nil,"BACKGROUND")
    divider.tex:SetColorTexture(1,1,1,0.35)
    divider.tex:Height(1)
    divider.tex:SetAllPoints()

    return divider
end

local function AddDivider(self)
    self:AddLine(" ")

    local line = self:NumLines()
    local relativeTo = _G[self:GetName().."TextLeft"..line]
    local divider = self.divider[line] or self:CreateDivider(line)

    divider:ClearAllPoints()
    divider:Point("RIGHT",-10,0)
    divider:Point("LEFT",relativeTo,0,1)
    divider:Show()
end

local function SetPrevLineJustify(self,justify)
    local index = self:NumLines()
    local line = _G[self:GetName().."TextLeft"..index]

    self.justify[line] = justify
end

function IFTT:GameTooltip_OnUpdate(tooltip)
    if tooltip:IsForbidden() then return end
    if (tooltip.needRefresh and tooltip:GetAnchorType() == "ANCHOR_CURSOR") then
        tooltip:SetBackdropColor(0, 0, 0, 0.65)
        tooltip:SetBackdropBorderColor(0, 0, 0)
        tooltip.needRefresh = nil
    end
end

function IFTT:Hook_Reset(tooltip)
    if tooltip:IsForbidden() then return end
    if tooltip.justify then
        for line, justify in pairs(tooltip.justify) do
            line:SetJustifyH("LEFT")
        end

        tooltip.justify = {}
    end

    if tooltip.divider then
        for line, divider in pairs(tooltip.divider) do
            divider:Hide()
        end
    end
end

function IFTT:PLAYER_ENTERING_WORLD(event)
    local tooltips = {
        "GameTooltip",
        "IF_BrokerTooltip",
    }

    for _, name in pairs(tooltips) do
        local tooltip = _G[name]
        if tooltip then
            tooltip.divider = {}
            tooltip.justify = {}
            tooltip.CreateDivider = CreateDivider
            tooltip.AddDivider = AddDivider
            tooltip.SetPrevLineJustify = SetPrevLineJustify
			
			self:SetStyle(tooltip)
            self:HookScript(tooltip, "OnShow", "SetStyle")
            self:HookScript(tooltip, "OnHide", function() self:Hook_Reset(tooltip) end)
        end
    end

    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

function IFTT:SetStyle(tooltip)
    if tooltip:IsForbidden() then return end
    if InCombatLockdown() then
        return tooltip:Hide()
    end
	
    -- if tooltip == GameTooltip then
    for line, justify in pairs(tooltip.justify) do
        local width = tooltip:GetWidth()-20

        line:SetWidth(width)
        line:SetJustifyH(justify)
    end
    -- end
    tooltip.needRefresh = true
end

function IFTT:Initialize()
	self:HookScript(GameTooltip, "OnUpdate", "GameTooltip_OnUpdate")
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
end

E:RegisterModule(IFTT:GetName())